/*
  global variables
*/

var widgetID = (new Date()).getTime();
var command = null;
var debugMode = true;
var reloadCounter;
var lastLoaded = (new Date()).getTime();
var reloadLevel = 0;
var waitPeriod = [ 5, 10, 20, 60, 120, 240, 480, 720, 1440 ];
var timer = null;
var reservedSize = { width:300, height:300 };
var modules = new Array();
var localizedStrings = null;

function tr( text ) {
    if ( localizedStrings ) {
        if ( localizedStrings[ text ] ) {
            return localizedStrings[ text ];
        }
    }
    return text;
}

// debug
function DEBUG( message ) {
    if ( !debugMode ) return;
    var elem = document.getElementById( 'debugDiv' );
    if ( elem ) {
        elem.innerText = message;
    }
}

function forceReload( event ) {
    reloadCounter = null;
    load( document.getElementById( "modules" ).value );
}

function tryReload() {
    var current = (new Date()).getTime();
    if ( reloadCounter == null ) {
        reloadLevel = 0;
        reloadCounter = current + waitPeriod[ 0 ] * 1000 * 60;
    }
    DEBUG( "trying reloading " + reloadCounter + " / " + current + " : " + ( ( reloadCounter - current ) / 1000 ) + " " + reloadLevel );

    if ( reloadCounter > current ) return;
    load( document.getElementById( "modules" ).value );
    reloadLevel ++;
    if ( reloadLevel >= waitPeriod.length ) {
        reloadLevel = waitPeriod.length - 1;
    }
    reloadCounter = current + waitPeriod[ reloadLevel ] * 1000 * 60;
    
}

function stripItem( str ) {
    var start = 0;
    var end = str.length;
    if ( str.length < 2 ) return str;
    if ( str.charAt( 0 ) == '"' ) start = 1;
    if ( str.charAt( str.length - 1 ) == '"' ) end = str.length - 1;
    return str.substring( start, end );
}

function __strip_item( str ) {
    var start = 0;
    var end = str.length;
    if ( str.length < 2 ) return str;
    if ( str.charAt( 0 ) == '"' ) start = 1;
    if ( str.charAt( str.length - 1 ) == '"' ) end = str.length - 1;
    return str.substring( start, end );
}

function DSModule( line ) {
    this.available = false;
    this.isAvailable = DSModule__isAvailable;
    var items     = line.split( ',' );
    if ( items.length < 5 ) return;
    this.data     = line;
    this.name     = __strip_item( items[ 0 ] );
    this.label    = __strip_item( items[ 1 ] );
    this.sports   = __strip_item( items[ 2 ] );
    this.language = __strip_item( items[ 3 ] );
    this.country  = __strip_item( items[ 4 ] );
    this.available = true;
}

function DSModule__isAvailable() {
    return this.available;
}

function listModules( country, language, sports ) {
    var select = document.getElementById( "modules" );
    if ( select ) {
        select.innerHTML = "";
        if ( country == undefined || country == "all" ) country = null;
        if ( language == undefined || language == "all" ) language = null;
        if ( sports == undefined || sports == "all" ) sports = null;
        for ( var i = 0; i < modules.length; i++ ) {
            var module = modules[ i ];
            if ( country ) {
                if ( module.country != country ) continue;
            }
            if ( language ) {
                if ( module.language != language ) continue;
            }
            if ( sports ) {
                if ( module.sports != sports ) continue;
            }
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', module.name );
            option.appendChild( document.createTextNode( module.label ) );
            select.appendChild( option );
        }
    }
}

function changeModuleList() {
    var country = null;
    var language = null;
    var sports = null;
    var elem_c = document.getElementById( "country" );
    var elem_l = document.getElementById( "language" );
    var elem_s = document.getElementById( "sports" );
    if ( elem_c ) {
        country = elem_c.value;
    }
    if ( elem_l ) {
        language = elem_l.value;
    }
    if ( elem_s ) {
        sports = elem_s.value;
    }
    //DEBUG( country + " " + language + " " + sports + " " );
    listModules( country, language, sports );
}

function listCountries() {
    var countries = new Array();
    for ( var i = 0; i < modules.length; i++ ) {
        var country = modules[ i ].country;
        var flag_exists = false;
        for ( var j = 0; j < countries.length; j++ ) {
            if ( countries[ j ] == country ) {
                flag_exists = true;
                break;
            }
        }
        if ( !flag_exists ) {
            countries.push( country );
        }
    }

    var elem_c = document.getElementById( "country" );
    if ( elem_c ) {
        elem_c.innerHTML = "<option value=\"all\">" + tr( "All" ) + "</option>";
        for ( var i = 0; i < countries.length; i++ ) {
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', countries[ i ] );
            option.appendChild( document.createTextNode( tr( countries[ i ] ) ) );
            elem_c.appendChild( option );
        }
    }
}

function listLanguages() {
    var items = new Array();
    for ( var i = 0; i < modules.length; i++ ) {
        var item = modules[ i ].language;
        var flag_exists = false;
        for ( var j = 0; j < items.length; j++ ) {
            if ( items[ j ] == item ) {
                flag_exists = true;
                break;
            }
        }
        if ( !flag_exists ) {
            items.push( item );
        }
    }

    var elem_c = document.getElementById( "language" );
    if ( elem_c ) {
        elem_c.innerHTML = "<option value=\"all\">" + tr( "All" ) + "</option>";
        for ( var i = 0; i < items.length; i++ ) {
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', items[ i ] );
            var text = tr( items[ i ] );
            option.appendChild( document.createTextNode( text ) );
            elem_c.appendChild( option );
        }
    }
}
function listSports() {
    var items = new Array();
    for ( var i = 0; i < modules.length; i++ ) {
        var item = modules[ i ].sports;
        var flag_exists = false;
        for ( var j = 0; j < items.length; j++ ) {
            if ( items[ j ] == item ) {
                flag_exists = true;
                break;
            }
        }
        if ( !flag_exists ) {
            items.push( item );
        }
    }

    var elem = document.getElementById( "sports" );
    if ( elem ) {
        elem.innerHTML = "<option value=\"all\">" + tr( "All" ) + "</option>";
        for ( var i = 0; i < items.length; i++ ) {
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', items[ i ] );
            var text = tr( items[ i ] );
            option.appendChild( document.createTextNode( text ) );
            elem.appendChild( option );
        }
    }
}

function refreshModules( args ) {
    if ( command != null ) {
        var output = command.outputString;
        mlist = output.split( "\n" );
        var select = document.getElementById( "modules" );
        
        for ( var i = 0; i < mlist.length; i++ ) {
            DEBUG( mlist[ i ] );
            var module = new DSModule( mlist[ i ] );
            if ( module.isAvailable() == false ) continue;
            modules.push( module );
        }
        listModules( null, null, null );
        listCountries();
        listLanguages();
        listSports();
//             //var items = mlist[ i ].split( "," );
//             //if ( items.length < 3 ) continue;

//             var option = document.createElement( 'option' );
//             option.setAttribute( 'value', module.name );//stripItem( items[ 0 ] ) );
//             option.appendChild( document.createTextNode( module.label ) );//stripItem( items[ 1 ] ) ) );
//             select.appendChild( option );
//         }
        command = null;
    }
}

function setup() {
    if ( window.widget ) {
        command = widget.system( "/usr/bin/python loader.py", refreshModules );
        widget.onremove = cleanup;

        createGenericButton( document.getElementById( 'done' ), tr( "Done" ), hidePreference );
    }
    document.getElementById( 'label-country' ).innerText = tr( 'Country' );
    document.getElementById( 'label-language' ).innerText = tr( 'Language' );
    document.getElementById( 'label-sports' ).innerText = tr( 'Sports' );
    document.getElementById( 'label-module' ).innerText = tr( 'Module' );
}

function endHandler(arg ) {
    var output = command.outputString;
//    DEBUG( output );
    var iframe = document.getElementById( "score" );
    if ( iframe ) {
        iframe.src = output;
//         var computedStyle = document.defaultView.getComputedStyle( iframe );
//         var width  = computedStyle.getPropertyValue( "width" );
//         var height = computedStyle.getPropertyValue( "height" );
//         DEBUG( "( " + width + ", " + height + " ) " );
    }
    command = null;
}

function changeModule(event) {
    var select = document.getElementById( "modules" );
    DEBUG( select.value );
}

function changed(event,elem) {
    load( event, elem );
}

function load( modulename ) {
    var value = modulename;
    if ( value == null || value == "" ) return;
    if ( window.widget ) {
        var pycommand = "/usr/bin/python loader.py \"" + value + "\" " + widgetID;
        DEBUG( pycommand );
        if ( command != null ) command.cancel();
        var iframe = document.getElementById( 'score' );
        if ( iframe ) {
            iframe.src = "Loading.html";
        }

        command = widget.system( pycommand, endHandler );
    }
}

function cleanup() {
    if ( window.widget ) {
        widget.system( "/usr/bin/python cleanup.py " + widgetID, null );
    }
}




var lastPos; 
function resizeMouseDown(event) 
{ 
    var x = event.x + window.screenX; 
    var y = event.y + window.screenY; 
    document.addEventListener("mousemove", resizeMouseMove, true); 
    document.addEventListener("mouseup", resizeMouseUp, true); 
    lastPos = {x:x, y:y}; 
    event.stopPropagation(); 
    event.preventDefault(); 
} 

function resizeMouseMove(event) 
{ 
    var screenX = event.x + window.screenX; 
    var screenY = event.y + window.screenY; 
    window.resizeBy(screenX - lastPos.x, screenY - lastPos.y); 
    lastPos = {x:screenX, y:screenY}; 
    event.stopPropagation(); 
    event.preventDefault(); 
} 

function resizeMouseUp(event) 
{ 
    document.removeEventListener("mousemove", resizeMouseMove, true); 
    document.removeEventListener("mouseup", resizeMouseUp, true); 
    event.stopPropagation(); 
    event.preventDefault(); 
} 

function flipToPreferencePanel( event ) {
//function showPreference(event) {
//    DEBUG( "pref" );
    var front = document.getElementById( "front" ); 
    var back  = document.getElementById( "behind" ); 
    var computedStyle = document.defaultView.getComputedStyle( front );
    var resized = false;
    reservedSize.width  = parseInt( computedStyle.getPropertyValue( "width" ) );
    reservedSize.height = parseInt( computedStyle.getPropertyValue( "height" ) );
    //DEBUG( "reserved " + reservedSize.width + ", " + reservedSize.height + " / " + window.size );
    if ( isLarger( 300, 200, reservedSize.width, reservedSize.height ) ) {
        window.resizeTo( 300, 200 );
        resized = true;
    }
    if (window.widget) {
        widget.prepareForTransition("ToBack"); 
    }
    if ( !resized ) {
        window.resizeTo( 300, 200 );
        resized = true;
    }

    back.style.display  = "block"; 
    front.style.display = "none"; 

    if (window.widget) {
        setTimeout ('widget.performTransition();', 0);  
    }
}

function isLarger( w1, h1, w2, h2 ) {
    if ( w1 > w2 || h1 > h2 ) {
        return true;
    }
    return false;
}

function hidePreference( event ) { 
    var front = document.getElementById("front"); 
    var back = document.getElementById("behind"); 
    var computedStyle = document.defaultView.getComputedStyle( back );
    var resized = false;
    var currentSize = { width:parseInt( computedStyle.getPropertyValue( "width" ) ), height:parseInt( computedStyle.getPropertyValue( "height" ) ) };

    DEBUG( "revert to " + reservedSize.width + ", " + reservedSize.height );
//    DEBUG( reservedSize );
    if ( isLarger( reservedSize.width, reservedSize.height, currentSize.width, currentSize.height ) ) {
        window.resizeTo( reservedSize.width, reservedSize.height );
        resized = true;
    }
    if (window.widget) {
        widget.prepareForTransition("ToFront"); 
    }
    if ( resized == false ) {
        window.resizeTo( parseInt( reservedSize.width ), 
                         parseInt( reservedSize.height ) );
    }
    front.style.display="block"; 
    back.style.display="none"; 
    if (window.widget) {
        setTimeout ('widget.performTransition();', 0);  
    }
    var select = document.getElementById( "modules" );
    if ( select && select.value) {
        reloadCounter = null;
        if ( timer != null ) {
            clearInterval( timer );
            timer = null;
        }
        timer = setInterval( tryReload, 1000 );
        load( select.value );
    }
} 

function pushDoneButton( event ) {
}

function revertDoneButton( event ) {
}

function enterFlipIcon(event) {
    document.getElementById( "flip" ).style.display = "block";
//    DEBUG( "enter icon" );
}

function exitFlipIcon(event) {
//    DEBUG( "exit icon" );
    document.getElementById( "flip" ).style.display = "none";
//     document.getElementById( "flip" ).style.display = "none";
}

// function hideBack() 
// { 
//     var front = document.getElementById("front"); 
//     var back = document.getElementById("back"); 
//     if (window.widget) 
//         widget.prepareForTransition("ToFront"); 
//     back.style.display="none"; 
//     front.style.display="block"; 
//     if (window.widget) 
//         setTimeout ('widget.performTransition();', 0); 
// } 


// var flipShown = false; 
// var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, 
// firstElement:null, timer:null}; 
// function mousemove (event) 
// { 
//     if (!flipShown) 
//     { 
//         if (animation.timer != null) 
//         { 
//             clearInterval (animation.timer); 
//             animation.timer  = null; 
//         } 
//         var starttime = (new Date).getTime() - 13; 
//         animation.duration = 500; 
//         animation.starttime = starttime; 
//         animation.firstElement = document.getElementById ('flip'); 
//         animation.timer = setInterval ("animate();", 13); 
//         animation.from = animation.now; 
//         animation.to = 1.0; 
//         animate(); 
//         flipShown = true; 
//     } 
// } 
// function mouseexit (event) 
// { 
//     if (flipShown) 
//     { 
//         // fade in the info button 
//         if (animation.timer != null) 
//         { 
//             clearInterval (animation.timer); 
//             animation.timer  = null; 
//         } 
//         var starttime = (new Date).getTime() - 13; 
//         animation.duration = 500; 
//         animation.starttime = starttime; 
//         animation.firstElement = document.getElementById ('flip'); 
//         animation.timer = setInterval ("animate();", 13); 
//         animation.from = animation.now; 
//         animation.to = 0.0; 
//         animate(); 
//         flipShown = false; 
//     } 
// } 

// function animate() 
// { 
//     var T; 
//     var ease; 
//     var time = (new Date).getTime(); 
//     T = limit_3(time-animation.starttime, 0, animation.duration); 
//     if (T >= animation.duration) 
//     { 
//         clearInterval (animation.timer); 
//         animation.timer = null; 
//         animation.now = animation.to; 
//     } 
//     else 
//     { 
//         ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration)); 
//         animation.now = computeNextFloat (animation.from, animation.to, ease); 
//     } 
//     animation.firstElement.style.opacity = animation.now; 
// } 

// function limit_3 (a, b, c) 
// { 
//     return a < b ? b : (a > c ? c : a); 
// } 
// function computeNextFloat (from, to, ease) 
// { 
//     return from + (to - from) * ease; 
// } 

// function enterFlipper(event) 
// { 
//     document.getElementById('fliprollie').style.display = 'block'; 
// } 
// function exitflip(event) 
// { 
//     document.getElementById('fliprollie').style.display = 'none'; 
// } 

