#!/usr/bin/python
# -*- coding:utf-8 -*-

"""JFA (Japan Football Association) recent results
"""

from dom import *
import scorecard

def newInstance():
    return JFANews_2005()


class JFANews_2005( scorecard.ScoreCard ):
    def __init__( self ):
        self.tables        = []
        self.headerclasses = []
        return
    
    def getSportsCategory( self ):
        return "soccer"

    def getCountry( self ):
        return "Japan"

    def getName( self ):
        return "JFA News (jfa.or.jp)"

    def getVersion( self ):
        return "1.0"

    def getEncoding( self ):
        return 'sjis'

    def getLanguage( self ):
        return "ja"

    def decode( self, contents ):
        import re, string
        contents = re.sub( "\"height=", "\" height=", contents, re.I )
        return unicode( contents, self.getEncoding() ).encode( 'utf-8' )
        
    def getURL( self ):
        return "http://www.jfa.or.jp/CGI/top/top.cgi?key=top"

    def __add_result( self, title, results ):
        import re, sys
        place = ""
        games = []
        game = { 'score':None, 'teams':None, 'detail':[] }
        for line in results:
            line = unicode( line, 'utf-8' )
            line = line.strip( u' \n\r　' )
            if line == "": continue
            if re.match( u'▼会場：', line ):
                place = line[ 4: ].encode( 'utf-8' )
                continue
            m = re.search( u'(.*?)(\\d+)\\s*\-\\s*(\\d+)(.*)', line )
            if m:
                games.append( game )
                game = { 'score':None, 'teams':None, 'detail':[] }
                team1 = m.group( 1 )
                ts1 = string.atoi( m.group( 2 ) )
                ts2 = string.atoi( m.group( 3 ) )
                m_first_half = re.search( u'(\\d+)\\s*\-\\s*(\\d+)(.*)', m.group( 4 ) )
                if m_first_half:
                    team2 = m_first_half.group( 3 )
                    fs1 = string.atoi( m_first_half.group( 1 ) )
                    fs2 = string.atoi( m_first_half.group( 2 ) )
                    game[ 'score' ] =( ( fs1, fs2 ), ( ts1 - fs1, ts2 - fs2 ), ( ts1, ts2 ) ) 
                else:
                    team2 = m.group( 4 )
                    game[ 'score' ] = ( ( ts1, ts2 ), )
                    pass
                game[ 'teams' ] = ( team1.strip( u' 　）').encode( 'utf-8' ), team2.strip( u' 　）' ).encode( 'utf-8' ) )
                pass
            elif line != "" and game != None:
                game[ 'detail' ].append( line.encode( 'utf-8' ) )
                pass
            pass
        if game != None:
            games.append( game )
            pass
        table = {'title':title, 'data': games }
        self.tables.append( table )
        return
        
    
    def interpreteDocument( self, document ):
        self.tables = []
        elem_game = document.get_element_at_path( "/base/html/body/div/table/tr/td/table" )
        while elem_game:
            title = ""
            result = None
            flag_title = False
            #print elem_game
            for child in elem_game.get_children():
                if not isinstance( child, Element ): continue
                if flag_title == False:
                    title += child.get_text()
                    if re.search( "\\d/\\d+/\\d+", title ): flag_title = True
                    pass
                else:
                    results = child.get_text( '-*-' ).split( '-*-' )
                    if len( results ) > 2:
                        self.__add_result( title, results )
                        pass
                    #break
                pass
            try:
                elem_game = elem_game.next_sibling()
            except:
                break
            pass

        pass
    

    def __create_row( self, teams, scores ):
        if len( scores ) == 3:
            row = '<tr><td class="team" rowspan="3">%s</td><td class="score">%d - %d</td><td class="team" rowspan="3">%s</td></tr>\n' % ( teams[ 0 ], scores[ 2 ][ 0 ], scores[ 2 ][ 1 ], teams[ 1 ] )
            for s in scores[ 0: 2 ]:
                row += '<tr><td class="score-half">%d - %d</td></tr>\n' % ( s[ 0 ], s[ 1 ] )
                pass
            return row
            pass
        else:
            return '<tr><td class="team">%s</td><td class="score">%d - %d</td><td class="team">%s</td></tr>\n' % ( teams[ 0 ], scores[ 0 ][ 0 ], scores[ 0 ][ 1 ], teams[ 1 ] )
        pass
        
    def toHTML( self ):
        html  = self.html_header()
        if len( self.tables ) == 0:
            html += self.html_footer()
            return html
        import sys
        #print len( self.tables )
        for table in self.tables:
            try:
                title = table[ 'title' ]
                data = table[ 'data' ]
                html += '<span class="tabletitle">%s</span>\n' % title
                html += "<table>\n"
                for game in data:
                    if game[ 'teams' ] != None and game[ 'score' ] != None:
                        html += self.__create_row( game[ 'teams' ], game[ 'score' ] )
                        pass
                    if game[ 'detail' ] != None and len( game[ 'detail' ] ) > 0:
                        for line in game[ 'detail' ]:
                            html += "<tr><td class=\"detail\" colspan=\"3\">%s</td></tr>\n" % line
                            pass
                        pass
                    pass
                html += "</table>\n"
                pass
            except:
                raise
                break
            pass
        html += self.html_footer()
        return html

    def html_css_plugin( self ):
        return """
        <style type=\"text/css\">
        table { margin-bottom:8px; }
        .score { text-align:center; valign:center;height:12px;font-weight:bold;}
        .score-half { color:gray; height:10px; font-size:9px; margin:0px; valign:center;padding:0px; text-align:center;}
        .detail {font-size:10px; margin:0px; text-align:center; height:12px;}
        .team {width:80px; height:12px;}
        
        </style>
        """
    
    def getStartKeyword( self ):
        return None

    def getEndKeyword( self ):
        return None

    pass


if __name__ == "__main__":
    """unit test"""
    filename_sample = None 
    import sys
    from loader import *

    if len( sys.argv ) > 1: filename_sample = sys.argv[ 1 ]
    obj = newInstance()

    if obj != None and obj.isAvailable():
        url = obj.getURL()
        if filename_sample:
            data = loadHTML_test( filename_sample )
        else:
            data = loadHTML( url )
            pass
        kw_start = obj.getStartKeyword()
        kw_end   = obj.getEndKeyword()
        parser = DOMConverter()
        if kw_start == None or kw_end == None:
            for line in data.split( "\n" ):
                parser.feed( obj.decode( line ) )
                pass
            pass
        else:
            flag_inside = False
            parser.feed( '<html>' )
            for line in re.split( "\n", data ):
                line = obj.decode( line.strip() )
                if flag_inside:
                    if line.count( kw_end ): flag_inside = False
                    else: parser.feed( line )
                elif line.count( kw_start ):
                    flag_inside = True
                    pass
                pass
            parser.feed( '</html>' )
            pass
        document = parser.get_document()
        obj.interpreteDocument( document )
        print obj.toHTML()
        pass
    pass
