#!/usr/bin/python
# -*- coding:utf-8 -*-

"""An abstract class of scoreboard classes.
"""

from dom import *

def newInstance():
    """This method must be inplemented in subclasses which instanciate
    an object."""
    return None


class ScoreCard:
    def __init__( self ):
        self.tables        = []
        self.headerclasses = []
        return

    """Overridable methods"""
    
    def getSportsCategory( self ):
        """Category of sports such as baseball, football, etc."""
        raise TypeError( "not implemented" )

    def getCountry( self ):
        """The country games are held."""
        raise TypeError( "not implemented" )

    def getName( self ):
        """Name of the module. This method must be overidden by
        any subclass.
        """
        return None

    def getVersion( self ):
        """Version of the mudule."""
        return "0.0"

    def getIcon( self, team ):
        """URI (local resource is recommended, see makelogo.py) of
        a team logo.
        This method returns empty string (not None) if no URI are
        found."""
        return ""

    def getEncoding( self ):
        """Texts are handled as UTF-8 in the programs. Then all
        texts must be encoded to the character code.
        This method helps the encoding process."""
        return 'latin1'

    def getLanguage( self ):
        """Language of the target website."""
        return "en"

    def decode( self, contents ):
        """Decoding method of given texts to UTF-8."""
        return unicode( contents, self.getEncoding() ).encode( 'utf-8' )
        
    def getURL( self ):
        """Refering URL. This method must be overridden."""
        raise TypeError( "not implemented" )

    def interpreteDocument( self, document ):
        """Interpreting information. This method must be overridden."""
        raise TypeError( "not implemented" )

    def getStartKeyword( self ):
        """For shortcutting HTML parsing, this method gives the
        keyword of start position of information.
        This method returns None when any keyword is required.
        """
        return None

    def getEndKeyword( self ):
        """For shortcutting HTML parsing, this method gives the
        keyword of end position of information.
        This method returns None when any keyword is required."""
        return None

    def toHTML( self ):
        """Format data into HTML.
        This method must be overridden."""
        html  = self.html_header()
        if len( self.tables ) == 0:
            html += self.html_footer()
            return html
        for table in self.tables:
            try:
                title = table[ 'title' ]
                rows = table[ 'data' ]
                html += '<span class="tabletitle">%s</span>\n' % title
                html += '<table>\n'
                for row in rows:
                    html += '<tr>'
                    for i in range( min( len( self.headerclasses ), len( row ) ) ):
                        html += '<td class="%s">%s</td>' % ( self.headerclasses[ i ], row[ i ] )
                        pass
                    html += '</tr>\n'
                    pass
                html += '</table>\n'
            except:
                pass
            pass
        html += self.html_footer()
        return html

    def html_css_plugin( self ):
        """This method is overidable by subclasses which uses stylesheet template and add some features in css."""
        return ""

    """Following methods are not supporsed for overiding."""
    def isAvailable( self ):
        """Check a subclass is valid.
        This method may not be overridden by subclasses."""
        try:
            #self.getURL()
            self.getSportsCategory()
        except:
            return False
        if self.getName() == None: return False
        return True


    def writeHTML( self, filename ):
        """Writing a HTML document to a file.
        This method is overridable if post-processing procedures are
        necessary.
        """
        file = open( filename, "w" )
        file.write( self, toHTML() )
        file.close()
        pass
        
    def xml_lang( self ):
        return "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"%s\" lang=\"%s\">" % ( self.getLanguage() , self.getLanguage() )

    def xml_header( self ):
        return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"

    def xml_doctype( self ):
        return "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
        
    def html_header( self ):
        html =  self.xml_header()
        html += self.xml_doctype()
        html += self.xml_lang()
        html += "<head><meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\"/>\n"
        html += "<title>%s</title>\n" % ( self.getName() )
        html += self.html_css()
        html += self.html_css_plugin()
        html += "</head>\n"
        return html

    def __getCSSPath( self, skin ):
        import os, os.path
        if skin == None or skin == "" or os.path.exists( '../%s.css' % skin ) == False:
            skin = 'score'
            pass
        uri = "file://" + os.path.abspath( os.path.curdir + "/.." ) + "/%s.css" % skin
        return uri

    def html_css( self, skin = None ):
        if skin == None: skin = 'score'
        return '<style type="text/css">\n  @import "%s";\n</style>\n' \
               % self.__getCSSPath( skin )
        return """
        <style type=\"text/css\">
        </style>
        """

    def html_footer( self ):
        return '</html>'

    pass


if __name__ == "__main__":
    """unit test"""
    filename_sample = None 
    import sys
    from loader import *

    if len( sys.argv ) > 1: filename_sample = sys.argv[ 1 ]
    
    obj = newInstance()

    if obj != None and obj.isAvailable():
        url = obj.getURL()
        if filename_sample:
            data = loadHTML_test( filename_sample )
        else:
            data = loadHTML( url )
            pass
        kw_start = obj.getStartKeyword()
        kw_end   = obj.getEndKeyword()
        parser = DOMConverter()
        if kw_start == None or kw_end == None:
            for line in data.split( "\n" ):
                parser.feed( obj.decode( line ) )
                pass
            pass
        else:
            flag_inside = False
            parser.feed( '<html>' )
            for line in re.split( "\n", data ):
                line = obj.decode( line.strip() )
                if flag_inside:
                    if line.count( kw_end ): flag_inside = False
                    else: parser.feed( line )
                elif line.count( kw_start ):
                    flag_inside = True
                    pass
                pass
            parser.feed( '</html>' )
            pass
        document = parser.get_document()
        obj.interpreteDocument( document )
        print obj.toHTML()
        pass
    pass
